@@ -52,6 +52,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #define STATUS_CONNECTING 0
 #define STATUS_CONNACK_RECVD 1
+#define STATUS_WAITING 2
 
 /* Global variables for use in callbacks. See sub_client.c for an example of
  * using a struct to hold variables for use in callbacks. */
@@ -63,6 +64,7 @@ static int retain = 0;
 static int mode = MSGMODE_NONE;
 static int status = STATUS_CONNECTING;
 static int mid_sent = 0;
+static int last_mid = -1;
 static bool connected = true;
 static char *username = NULL;
 static char *password = NULL;
@@ -123,7 +125,12 @@ void my_disconnect_callback(struct mosquitto *mosq, void *obj, int rc)
 
 void my_publish_callback(struct mosquitto *mosq, void *obj, int mid)
 {
-	if(mode != MSGMODE_STDIN_LINE && disconnect_sent == false){
+	if(mode == MSGMODE_STDIN_LINE){
+		if(mid == last_mid){
+			mosquitto_disconnect(mosq);
+			disconnect_sent = true;
+		}
+	}else if(disconnect_sent == false){
 		mosquitto_disconnect(mosq);
 		disconnect_sent = true;
 	}
@@ -202,7 +209,11 @@ int load_file(const char *filename)
 
 void print_usage(void)
 {
-	printf("mosquitto_pub is a simple mqtt client that will publish a message on a single topic and exit.\n\n");
+	int major, minor, revision;
+
+	mosquitto_lib_version(&major, &minor, &revision);
+	printf("mosquitto_pub is a simple mqtt client that will publish a message on a single topic and exit.\n");
+	printf("mosquitto_pub version %s running on libmosquitto %d.%d.%d.\n\n", VERSION, major, minor, revision);
 	printf("Usage: mosquitto_pub [-h host] [-p port] [-q qos] [-r] {-f file | -l | -n | -m message} -t topic\n");
 	printf("                     [-i id] [-I id_prefix]\n");
 	printf("                     [-d] [--quiet]\n");
@@ -262,9 +273,6 @@ int main(int argc, char *argv[])
 	char *host = "localhost";
 	int port = 1883;
 	int keepalive = 60;
-#ifndef WIN32
-	int opt;
-#endif
 	char buf[1024];
 	bool debug = false;
 	struct mosquitto *mosq = NULL;
@@ -401,13 +409,6 @@ int main(int argc, char *argv[])
 				return 1;
 			}else{
 				mode = MSGMODE_STDIN_LINE;
-#ifndef WIN32
-				opt = fcntl(fileno(stdin), F_GETFL, 0);
-				if(opt == -1 || fcntl(fileno(stdin), F_SETFL, opt | O_NONBLOCK) == -1){
-					fprintf(stderr, "Error: Unable to set stdin to non-blocking.\n");
-					return 1;
-				}
-#endif
 			}
 		}else if(!strcmp(argv[i], "-m") || !strcmp(argv[i], "--message")){
 			if(mode != MSGMODE_NONE){
@@ -599,7 +600,10 @@ int main(int argc, char *argv[])
 			return 1;
 		}
 		snprintf(id, len, "mosqpub/%d-%s", getpid(), hostname);
-		id[MOSQ_MQTT_ID_MAX_LENGTH] = '\0';
+		if(strlen(id) > MOSQ_MQTT_ID_MAX_LENGTH){
+			/* Enforce maximum client id length of 23 characters */
+			id[MOSQ_MQTT_ID_MAX_LENGTH] = '\0';
+		}
 	}
 
 	mosq = mosquitto_new(id, true, NULL);
@@ -660,23 +664,41 @@ int main(int argc, char *argv[])
 		return rc;
 	}
 
+	if(mode == MSGMODE_STDIN_LINE){
+		mosquitto_loop_start(mosq);
+	}
+
 	do{
-		if(mode == MSGMODE_STDIN_LINE && status == STATUS_CONNACK_RECVD){
-			if(fgets(buf, 1024, stdin)){
-				buf[strlen(buf)-1] = '\0';
-				rc2 = mosquitto_publish(mosq, &mid_sent, topic, strlen(buf), buf, qos, retain);
-				if(rc2){
-					if(!quiet) fprintf(stderr, "Error: Publish returned %d, disconnecting.\n", rc2);
-					mosquitto_disconnect(mosq);
+		if(mode == MSGMODE_STDIN_LINE){
+			if(status == STATUS_CONNACK_RECVD){
+				if(fgets(buf, 1024, stdin)){
+					buf[strlen(buf)-1] = '\0';
+					rc2 = mosquitto_publish(mosq, &mid_sent, topic, strlen(buf), buf, qos, retain);
+					if(rc2){
+						if(!quiet) fprintf(stderr, "Error: Publish returned %d, disconnecting.\n", rc2);
+						mosquitto_disconnect(mosq);
+					}
+				}else if(feof(stdin)){
+					last_mid = mid_sent;
+					status = STATUS_WAITING;
 				}
-			}else if(feof(stdin) && disconnect_sent == false){
-				mosquitto_disconnect(mosq);
-				disconnect_sent = true;
+			}else if(status == STATUS_WAITING){
+#ifdef WIN32
+				Sleep(1000);
+#else
+				usleep(1000000);
+#endif
 			}
+			rc = MOSQ_ERR_SUCCESS;
+		}else{
+			rc = mosquitto_loop(mosq, -1, 1);
 		}
-		rc = mosquitto_loop(mosq, -1, 1);
 	}while(rc == MOSQ_ERR_SUCCESS && connected);
 
+	if(mode == MSGMODE_STDIN_LINE){
+		mosquitto_loop_stop(mosq, false);
+	}
+
 	if(message && mode == MSGMODE_FILE){
 		free(message);
 	}
