@@ -46,14 +46,14 @@ POSSIBILITY OF SUCH DAMAGE.
 
 namespace mosqpp {
 
-const char *strerror(int mosq_errno);
-const char *connack_string(int connack_code);
-int sub_topic_tokenise(const char *subtopic, char ***topics, int *count);
-int sub_topic_tokens_free(char ***topics, int count);
-int lib_version(int *major, int *minor, int *revision);
-int lib_init();
-int lib_cleanup();
-int topic_matches_sub(const char *sub, const char *topic, bool *result);
+mosqpp_EXPORT const char *strerror(int mosq_errno);
+mosqpp_EXPORT const char *connack_string(int connack_code);
+mosqpp_EXPORT int sub_topic_tokenise(const char *subtopic, char ***topics, int *count);
+mosqpp_EXPORT int sub_topic_tokens_free(char ***topics, int count);
+mosqpp_EXPORT int lib_version(int *major, int *minor, int *revision);
+mosqpp_EXPORT int lib_init();
+mosqpp_EXPORT int lib_cleanup();
+mosqpp_EXPORT int topic_matches_sub(const char *sub, const char *topic, bool *result);
 
 /*
  * Class: mosquittopp
@@ -81,7 +81,7 @@ class mosqpp_EXPORT mosquittopp {
 		int subscribe(int *mid, const char *sub, int qos=0);
 		int unsubscribe(int *mid, const char *sub);
 		void message_retry_set(unsigned int message_retry);
-		void user_data_set(void *obj);
+		void user_data_set(void *userdata);
 		int tls_set(const char *cafile, const char *capath=NULL, const char *certfile=NULL, const char *keyfile=NULL, int (*pw_callback)(char *buf, int size, int rwflag, void *userdata)=NULL);
 		int tls_opts_set(int cert_reqs, const char *tls_version=NULL, const char *ciphers=NULL);
 		int tls_psk_set(const char *psk, const char *identity, const char *ciphers=NULL);
@@ -90,6 +90,7 @@ class mosqpp_EXPORT mosquittopp {
 		int loop_misc();
 		int loop_read(int max_packets=1);
 		int loop_write(int max_packets=1);
+		int loop_forever(int timeout=-1, int max_packets=1);
 		int loop_start();
 		int loop_stop(bool force=false);
 		bool want_write();
