@@ -44,7 +44,7 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #ifdef WITH_BRIDGE
 
-int mqtt3_bridge_new(mosquitto_db *db, struct _mqtt3_bridge *bridge)
+int mqtt3_bridge_new(struct mosquitto_db *db, struct _mqtt3_bridge *bridge)
 {
 	int i;
 	struct mosquitto *new_context = NULL;
@@ -135,7 +135,7 @@ int mqtt3_bridge_new(mosquitto_db *db, struct _mqtt3_bridge *bridge)
 	return mqtt3_bridge_connect(db, new_context);
 }
 
-int mqtt3_bridge_connect(mosquitto_db *db, struct mosquitto *context)
+int mqtt3_bridge_connect(struct mosquitto_db *db, struct mosquitto *context)
 {
 	int rc;
 	int i;
@@ -156,15 +156,20 @@ int mqtt3_bridge_connect(mosquitto_db *db, struct mosquitto *context)
 	mqtt3_bridge_packet_cleanup(context);
 	mqtt3_db_message_reconnect_reset(context);
 
+	if(context->clean_session){
+		mqtt3_db_messages_delete(context);
+	}
+
+	/* Delete all local subscriptions even for clean_session==false. We don't
+	 * remove any messages and the next loop carries out the resubscription
+	 * anyway. This means any unwanted subs will be removed.
+	 */
+	mqtt3_subs_clean_session(db, context, &db->subs);
+
 	for(i=0; i<context->bridge->topic_count; i++){
 		if(context->bridge->topics[i].direction == bd_out || context->bridge->topics[i].direction == bd_both){
 			_mosquitto_log_printf(NULL, MOSQ_LOG_DEBUG, "Bridge %s doing local SUBSCRIBE on topic %s", context->id, context->bridge->topics[i].local_topic);
 			if(mqtt3_sub_add(db, context, context->bridge->topics[i].local_topic, context->bridge->topics[i].qos, &db->subs)) return 1;
-		}else{
-			/* direction = inwards only. This means we should not be subscribed
-			 * to the topic. It is possible that we used to be subscribed to
-			 * this topic so unsubscribe. */
-			mqtt3_sub_remove(db, context, context->bridge->topics[i].local_topic, &db->subs);
 		}
 	}
 
