@@ -59,15 +59,16 @@ extern SERVICE_STATUS_HANDLE service_handle;
 static int _conf_parse_bool(char **token, const char *name, bool *value, char *saveptr);
 static int _conf_parse_int(char **token, const char *name, int *value, char *saveptr);
 static int _conf_parse_string(char **token, const char *name, char **value, char *saveptr);
-static int _config_read_file(mqtt3_config *config, bool reload, const char *file, struct config_recurse *config_tmp, int level);
+static int _config_read_file(struct mqtt3_config *config, bool reload, const char *file, struct config_recurse *config_tmp, int level);
 
-static void _config_init_reload(mqtt3_config *config)
+static void _config_init_reload(struct mqtt3_config *config)
 {
 	int i;
 	/* Set defaults */
 	if(config->acl_file) _mosquitto_free(config->acl_file);
 	config->acl_file = NULL;
 	config->allow_anonymous = true;
+	config->allow_duplicate_messages = false;
 	config->autosave_interval = 1800;
 	config->autosave_on_changes = false;
 	if(config->clientid_prefixes) _mosquitto_free(config->clientid_prefixes);
@@ -112,9 +113,9 @@ static void _config_init_reload(mqtt3_config *config)
 	}
 }
 
-void mqtt3_config_init(mqtt3_config *config)
+void mqtt3_config_init(struct mqtt3_config *config)
 {
-	memset(config, 0, sizeof(mqtt3_config));
+	memset(config, 0, sizeof(struct mqtt3_config));
 	_config_init_reload(config);
 	config->config_file = NULL;
 	config->daemon = false;
@@ -147,7 +148,7 @@ void mqtt3_config_init(mqtt3_config *config)
 	config->auth_plugin = NULL;
 }
 
-void mqtt3_config_cleanup(mqtt3_config *config)
+void mqtt3_config_cleanup(struct mqtt3_config *config)
 {
 	int i, j;
 
@@ -227,7 +228,7 @@ static void print_usage(void)
 	printf("\nSee http://mosquitto.org/ for more information.\n\n");
 }
 
-int mqtt3_config_parse_args(mqtt3_config *config, int argc, char *argv[])
+int mqtt3_config_parse_args(struct mqtt3_config *config, int argc, char *argv[])
 {
 	int i;
 	int port_tmp;
@@ -327,11 +328,9 @@ int mqtt3_config_parse_args(mqtt3_config *config, int argc, char *argv[])
 	return MOSQ_ERR_SUCCESS;
 }
 
-int mqtt3_config_read(mqtt3_config *config, bool reload)
+int mqtt3_config_read(struct mqtt3_config *config, bool reload)
 {
 	int rc = MOSQ_ERR_SUCCESS;
-	int max_inflight_messages = 20;
-	int max_queued_messages = 100;
 	struct config_recurse cr;
 	int i;
 
@@ -377,7 +376,7 @@ int mqtt3_config_read(mqtt3_config *config, bool reload)
 		config->user = "mosquitto";
 	}
 
-	mqtt3_db_limits_set(max_inflight_messages, max_queued_messages);
+	mqtt3_db_limits_set(cr.max_inflight_messages, cr.max_queued_messages);
 
 #ifdef WITH_BRIDGE
 	for(i=0; i<config->bridge_count; i++){
@@ -397,7 +396,7 @@ int mqtt3_config_read(mqtt3_config *config, bool reload)
 	return MOSQ_ERR_SUCCESS;
 }
 
-int _config_read_file(mqtt3_config *config, bool reload, const char *file, struct config_recurse *cr, int level)
+int _config_read_file(struct mqtt3_config *config, bool reload, const char *file, struct config_recurse *cr, int level)
 {
 	int rc;
 	FILE *fptr = NULL;
@@ -478,6 +477,8 @@ int _config_read_file(mqtt3_config *config, bool reload, const char *file, struc
 #endif
 				}else if(!strcmp(token, "allow_anonymous")){
 					if(_conf_parse_bool(&token, "allow_anonymous", &config->allow_anonymous, saveptr)) return MOSQ_ERR_INVAL;
+				}else if(!strcmp(token, "allow_duplicate_messages")){
+					if(_conf_parse_bool(&token, "allow_duplicate_messages", &config->allow_duplicate_messages, saveptr)) return MOSQ_ERR_INVAL;
 				}else if(!strncmp(token, "auth_opt_", 9)){
 					if(strlen(token) < 12){
 						/* auth_opt_ == 9, + one digit key == 10, + one space == 11, + one value == 12 */
@@ -1173,11 +1174,6 @@ int _config_read_file(mqtt3_config *config, bool reload, const char *file, struc
 					config->default_listener.port = port_tmp;
 				}else if(!strcmp(token, "psk_file")){
 #if defined(WITH_TLS) && defined(WITH_TLS_PSK)
-					if(reload) continue; // Listeners not valid for reloading.
-					if(cur_listener->cafile || config->default_listener.capath){
-						_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Cannot use both certificate and psk encryption in a single listener.");
-						return MOSQ_ERR_INVAL;
-					}
 					if(reload){
 						if(config->psk_file){
 							_mosquitto_free(config->psk_file);
