@@ -33,6 +33,8 @@ POSSIBILITY OF SUCH DAMAGE.
 #ifndef WIN32
 #include <netdb.h>
 #include <unistd.h>
+#include <arpa/inet.h>
+#include <sys/socket.h>
 #else
 #include <winsock2.h>
 #include <ws2tcpip.h>
@@ -47,6 +49,11 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <tcpd.h>
 #endif
 
+#ifdef __FreeBSD__
+#  include <netinet/in.h>
+#  include <sys/socket.h>
+#endif
+
 #ifdef __QNX__
 #include <netinet/in.h>
 #include <net/netbyte.h>
@@ -60,11 +67,14 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <util_mosq.h>
 
 #ifdef WITH_TLS
+#include <openssl/ssl.h>
 static int tls_ex_index_context = -1;
 static int tls_ex_index_listener = -1;
 #endif
 
-int mqtt3_socket_accept(struct _mosquitto_db *db, int listensock)
+extern unsigned int g_socket_connections;
+
+int mqtt3_socket_accept(struct mosquitto_db *db, int listensock)
 {
 	int i;
 	int j;
@@ -78,11 +88,14 @@ int mqtt3_socket_accept(struct _mosquitto_db *db, int listensock)
 #endif
 #ifdef WITH_WRAP
 	struct request_info wrap_req;
+	char address[1024];
 #endif
 
 	new_sock = accept(listensock, NULL, 0);
 	if(new_sock == INVALID_SOCKET) return -1;
 
+	g_socket_connections++;
+
 #ifndef WIN32
 	/* Set non-blocking */
 	opt = fcntl(new_sock, F_GETFL, 0);
@@ -104,7 +117,9 @@ int mqtt3_socket_accept(struct _mosquitto_db *db, int listensock)
 	fromhost(&wrap_req);
 	if(!hosts_access(&wrap_req)){
 		/* Access is denied */
-		_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Client connection denied access by tcpd.");
+		if(!_mosquitto_socket_get_address(new_sock, address, 1024)){
+			_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Client connection from %s denied access by tcpd.", address);
+		}
 		COMPAT_CLOSE(new_sock);
 		return -1;
 	}else{
@@ -128,6 +143,7 @@ int mqtt3_socket_accept(struct _mosquitto_db *db, int listensock)
 		}
 
 		if(new_context->listener->max_connections > 0 && new_context->listener->client_count >= new_context->listener->max_connections){
+			_mosquitto_log_printf(NULL, MOSQ_LOG_NOTICE, "Client connection from %s denied: max_connections exceeded.", new_context->address);
 			COMPAT_CLOSE(new_sock);
 			return -1;
 		}
@@ -199,7 +215,7 @@ static int client_certificate_verify(int preverify_ok, X509_STORE_CTX *ctx)
 #if defined(WITH_TLS) && defined(WITH_TLS_PSK)
 static unsigned int psk_server_callback(SSL *ssl, const char *identity, unsigned char *psk, unsigned int max_psk_len)
 {
-	struct _mosquitto_db *db;
+	struct mosquitto_db *db;
 	struct mosquitto *context;
 	struct _mqtt3_listener *listener;
 	char *psk_key = NULL;
@@ -347,6 +363,14 @@ int mqtt3_socket_listen(struct _mqtt3_listener *listener)
 				COMPAT_CLOSE(sock);
 				return 1;
 			}
+#if OPENSSL_VERSION_NUMBER >= 0x10000000
+			/* Disable compression */
+			SSL_CTX_set_options(listener->ssl_ctx, SSL_OP_NO_COMPRESSION);
+#endif
+#ifdef SSL_MODE_RELEASE_BUFFERS
+			/* Use even less memory per SSL connection. */
+			SSL_CTX_set_mode(listener->ssl_ctx, SSL_MODE_RELEASE_BUFFERS);
+#endif
 			if(listener->ciphers){
 				rc = SSL_CTX_set_cipher_list(listener->ssl_ctx, listener->ciphers);
 				if(rc == 0){
@@ -450,3 +474,22 @@ int mqtt3_socket_listen(struct _mqtt3_listener *listener)
 	}
 }
 
+int _mosquitto_socket_get_address(int sock, char *buf, int len)
+{
+	struct sockaddr_storage addr;
+	socklen_t addrlen;
+
+	addrlen = sizeof(addr);
+	if(!getpeername(sock, (struct sockaddr *)&addr, &addrlen)){
+		if(addr.ss_family == AF_INET){
+			if(inet_ntop(AF_INET, &((struct sockaddr_in *)&addr)->sin_addr.s_addr, buf, len)){
+				return 0;
+			}
+		}else if(addr.ss_family == AF_INET6){
+			if(inet_ntop(AF_INET6, &((struct sockaddr_in6 *)&addr)->sin6_addr.s6_addr, buf, len)){
+				return 0;
+			}
+		}
+	}
+	return 1;
+}
