@@ -38,7 +38,9 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <send_mosq.h>
 #include <util_mosq.h>
 
-int mqtt3_handle_connect(mosquitto_db *db, struct mosquitto *context)
+extern unsigned int g_connection_count;
+
+int mqtt3_handle_connect(struct mosquitto_db *db, struct mosquitto *context)
 {
 	char *protocol_name;
 	uint8_t protocol_version;
@@ -60,6 +62,8 @@ int mqtt3_handle_connect(mosquitto_db *db, struct mosquitto *context)
 	X509_NAME_ENTRY *name_entry;
 #endif
 
+	g_connection_count++;
+
 	/* Don't accept multiple CONNECT commands. */
 	if(context->state != mosq_cs_new){
 		mqtt3_context_disconnect(db, context);
@@ -220,7 +224,19 @@ int mqtt3_handle_connect(mosquitto_db *db, struct mosquitto *context)
 		}else{
 #endif /* WITH_TLS_PSK */
 			client_cert = SSL_get_peer_certificate(context->ssl);
+			if(!client_cert){
+				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
+				mqtt3_context_disconnect(db, context);
+				_mosquitto_free(client_id);
+				return MOSQ_ERR_SUCCESS;
+			}
 			name = X509_get_subject_name(client_cert);
+			if(!name){
+				_mosquitto_send_connack(context, CONNACK_REFUSED_BAD_USERNAME_PASSWORD);
+				mqtt3_context_disconnect(db, context);
+				_mosquitto_free(client_id);
+				return MOSQ_ERR_SUCCESS;
+			}
 
 			i = X509_NAME_get_index_by_NID(name, NID_commonName, -1);
 			if(i == -1){
@@ -359,7 +375,7 @@ int mqtt3_handle_connect(mosquitto_db *db, struct mosquitto *context)
 	return _mosquitto_send_connack(context, CONNACK_ACCEPTED);
 }
 
-int mqtt3_handle_disconnect(mosquitto_db *db, struct mosquitto *context)
+int mqtt3_handle_disconnect(struct mosquitto_db *db, struct mosquitto *context)
 {
 	if(!context){
 		return MOSQ_ERR_INVAL;
@@ -374,7 +390,7 @@ int mqtt3_handle_disconnect(mosquitto_db *db, struct mosquitto *context)
 }
 
 
-int mqtt3_handle_subscribe(mosquitto_db *db, struct mosquitto *context)
+int mqtt3_handle_subscribe(struct mosquitto_db *db, struct mosquitto *context)
 {
 	int rc = 0;
 	int rc2;
@@ -438,24 +454,13 @@ int mqtt3_handle_subscribe(mosquitto_db *db, struct mosquitto *context)
 
 			}
 			_mosquitto_log_printf(NULL, MOSQ_LOG_DEBUG, "\t%s (QoS %d)", sub, qos);
-			/* FIXME - need to deny access to retained messages. */
-#if 0
-			/* Check for topic access */
-			rc2 = mqtt3_acl_check(db, context, sub, MOSQ_ACL_READ);
-			if(rc2 == MOSQ_ERR_SUCCESS){
-				mqtt3_sub_add(db, context, sub, qos, &db->subs);
-				if(mqtt3_retain_queue(db, context, sub, qos)) rc = 1;
-			}else if(rc2 != MOSQ_ERR_ACL_DENIED){
-				rc = 1;
-			}
-#else
+
 			rc2 = mqtt3_sub_add(db, context, sub, qos, &db->subs);
 			if(rc2 == MOSQ_ERR_SUCCESS){
 				if(mqtt3_retain_queue(db, context, sub, qos)) rc = 1;
 			}else if(rc2 != -1){
 				rc = rc2;
 			}
-#endif
 			_mosquitto_free(sub);
 
 			tmp_payload = _mosquitto_realloc(payload, payloadlen + 1);
@@ -481,7 +486,7 @@ int mqtt3_handle_subscribe(mosquitto_db *db, struct mosquitto *context)
 	return rc;
 }
 
-int mqtt3_handle_unsubscribe(mosquitto_db *db, struct mosquitto *context)
+int mqtt3_handle_unsubscribe(struct mosquitto_db *db, struct mosquitto *context)
 {
 	uint16_t mid;
 	char *sub;
