@@ -31,13 +31,11 @@ try:
     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     sock.connect(("localhost", 1888))
     sock.send(connect_packet)
-    connack_recvd = sock.recv(len(connack_packet))
 
-    if mosq_test.packet_matches("connack", connack_recvd, connack_packet):
+    if mosq_test.expect_packet(sock, "connack", connack_packet):
         sock.send(subscribe_packet)
-        suback_recvd = sock.recv(len(suback_packet))
 
-        if mosq_test.packet_matches("suback", suback_recvd, suback_packet):
+        if mosq_test.expect_packet(sock, "suback", suback_packet):
             rc = 0
 
     sock.close()
