@@ -43,13 +43,11 @@ client = subprocess.Popen(client_args, env=env)
 try:
     (conn, address) = sock.accept()
     conn.settimeout(10)
-    connect_recvd = conn.recv(len(connect_packet))
 
-    if mosq_test.packet_matches("connect", connect_recvd, connect_packet):
+    if mosq_test.expect_packet(conn, "connect", connect_packet):
         conn.send(connack_packet)
-        publish_recvd = conn.recv(len(publish_packet))
 
-        if mosq_test.packet_matches("publish", publish_recvd, publish_packet):
+        if mosq_test.expect_packet(conn, "publish", publish_packet):
             rc = 0
         
     conn.close()
